@@ -40,7 +40,6 @@ import smart_open.hdfs as smart_open_hdfs
 import smart_open.webhdfs as smart_open_webhdfs
 import smart_open.http as smart_open_http
 import smart_open.ssh as smart_open_ssh
-import smart_open.gcs as smart_open_gcs
 
 from smart_open import doctools
 
@@ -384,10 +383,6 @@ open.__doc__ = None if open.__doc__ is None else open.__doc__ % {
         doctools.extract_kwargs(smart_open_ssh.open.__doc__),
         lpad=u'    ',
     ),
-    'gcs': doctools.to_docstring(
-        doctools.extract_kwargs(smart_open_gcs.open.__doc__),
-        lpad=u'    ',
-    ),
     'examples': doctools.extract_examples_from_readme_rst(),
 }
 
@@ -583,9 +578,6 @@ def _open_binary_stream(uri, mode, transport_params):
             filename = P.basename(urlparse.urlparse(uri).path)
             kw = _check_kwargs(smart_open_http.open, transport_params)
             return smart_open_http.open(uri, mode, **kw), filename
-        elif parsed_uri.scheme == smart_open_gcs.SUPPORTED_SCHEME:
-            kw = _check_kwargs(smart_open_gcs.open, transport_params)
-            return smart_open_gcs.open(parsed_uri.bucket_id, parsed_uri.blob_id, mode, **kw), filename
         else:
             raise NotImplementedError("scheme %r is not supported", parsed_uri.scheme)
     elif hasattr(uri, 'read'):
@@ -748,8 +740,6 @@ def _parse_uri(uri_as_string):
         return Uri(scheme=parsed_uri.scheme, uri_path=uri_as_string)
     elif parsed_uri.scheme in smart_open_ssh.SCHEMES:
         return _parse_uri_ssh(parsed_uri)
-    elif parsed_uri.scheme == smart_open_gcs.SUPPORTED_SCHEME:
-        return _parse_uri_gcs(parsed_uri)
     else:
         raise NotImplementedError(
             "unknown URI scheme %r in %r" % (parsed_uri.scheme, uri_as_string)
@@ -846,13 +836,6 @@ def _unquote(text):
     return text and urlparse.unquote(text)
 
 
-def _parse_uri_gcs(parsed_uri):
-    assert parsed_uri.scheme == smart_open_gcs.SUPPORTED_SCHEME
-    bucket_id, blob_id = parsed_uri.netloc, parsed_uri.path[1:]
-
-    return Uri(scheme=parsed_uri.scheme, bucket_id=bucket_id, blob_id=blob_id)
-
-
 def _need_to_buffer(file_obj, mode, ext):
     """Returns True if we need to buffer the whole file in memory in order to proceed."""
     try:
